#!/usr/bin/env python

import argparse
import elitech
import datetime
from elitech.msg import (
    StopButton,
    ToneSet,
    AlarmSetting,
    TemperatureUnit,

)

def main():
    args = parse_args()
    if (args.command == 'simple-set'):
        command_simpleset(args)
    elif(args.command == 'get'):
        command_get(args)
    elif(args.command == 'set'):
        command_set(args)
    elif(args.command == 'devinfo'):
        command_devinfo(args)

def _convert_time(sec):
    hour = int(sec / 3600.0)
    min = int((sec - hour * 3600) / 60.0)
    sec = sec % 60
    return datetime.time(hour=hour, minute=min, second=sec)

def command_simpleset(args):
    device = elitech.Device(args.serial_port)
    device.init()
    dev_info = device.get_devinfo()
    param_put = dev_info.to_param_put()

    if args.interval:
        param_put.rec_interval = _convert_time(args.interval)
    device.update(param_put)

def command_get(args):
    device = elitech.Device(args.serial_port)
    device.init()

    def output(data_list):
        for line in data_list:
            print("{0}\t{1:%Y-%m-%d %H:%M:%S}\t{2:.1f}".format(*line))

    device.get_data(callback=output)

def command_set(args):
    device = elitech.Device(args.serial_port)
    device.init()
    dev_info = device.get_devinfo()
    param_put = dev_info.to_param_put()

    if args.interval:
        param_put.rec_interval = _convert_time(args.interval)
    if args.upper_limit:
        param_put.upper_limit = args.upper_limit
    if args.lower_limit:
        param_put.lower_limit = args.lower_limit
    if args.station_no:
        param_put.update_station_no = int(args.station_no)
    if args.stop_button:
        param_put.stop_button = StopButton.ENABLE if args.stop_button == 'y' else StopButton.DISABLE
    if args.delay:
        param_put.delay = float(args.delay)
    if args.tone_set:
        param_put.tone_set = ToneSet.PERMIT if args.tone_set == 'y' else ToneSet.NONE
    if args.alarm:
        if args.alarm == 'x':
            param_put.alarm = AlarmSetting.NONE
        elif args.alarm == '3':
            param_put.alarm = AlarmSetting.T3
        elif args.alarm == '10':
            param_put.alarm = AlarmSetting.T10
    if args.temp_unit:
        param_put.temp_unit = TemperatureUnit.C if args.temp_unit == 'C' else TemperatureUnit.F

    if args.temp_calibration:
        param_put.temp_calibration = float(args.temp_calibration)

    for k,v in vars(param_put).items():
        print("{}={}".format(k, v))


    device.update(param_put)

def command_devinfo(args):
    device = elitech.Device(args.serial_port)
    device.init()
    dev_info = device.get_devinfo()
    for k,v in vars(dev_info).items():
        print("{}={}".format(k, v))


def parse_args():
    """
    :rtype: argparse.Namespace
    """
    parser = argparse.ArgumentParser('description elitech RC-4 data reader')
    parser.add_argument('-c', "--command", choices=['init', 'get', 'simple-set', 'set', 'devinfo'])
    parser.add_argument('-i', "--interval", type=int)
    parser.add_argument("--upper_limit", type=float)
    parser.add_argument("--lower_limit", type=float)
    parser.add_argument("--station_no", type=int)
    parser.add_argument("--stop_button", choices=['y', 'n'])
    parser.add_argument("--delay", choices=['0.0', '0.5', '1.0', '1.5', '2.0', '2.5', '3.0', '3.5', '4.0', '4.5', '5.0', '5.5', '6.0'])
    parser.add_argument('--tone_set', choices=['y', 'n'])
    parser.add_argument('--alarm', choices=['x', '3', '10'])
    parser.add_argument('--temp_unit', choices=['C', 'F'])
    parser.add_argument('--temp_calibration', type=float)
    parser.add_argument('serial_port')
    return parser.parse_args()




if __name__ == '__main__':
    main()




__author__ = 'civic'

from struct import unpack, pack
from datetime import datetime, time
from enum import Enum
import six

def _bin(s):
    """
    :rtype: bytes
    """
    return bytes([int(b, 16) for b in s.split(" ")])

def _intarray2bytes(int_array):
    return bytes(int_array)

if six.PY2:
    def py2bin(s):
        return "".join([chr(int(b, 16)) for b in s.split(" ")])
    def py2intarray2bytes(int_array):
        return "".join([chr(b) for b in int_array])

    _bin = py2bin
    _intarray2bytes = py2intarray2bytes

def _datetime_unpack(date_bytes):
    """
    :rtype: datetime
    """
    return datetime(*unpack(">h5b", date_bytes))


def _interval_unpack(time_bytes):
    """
    :rtype: time
    """
    return time(*unpack(">3b", time_bytes))


def _interval_pack(t):
    """
    :param t: time.time
    :return: bytes
    """
    return _intarray2bytes([t.hour, t.minute, t.second])


def _append_checksum(byte_array):
    if six.PY2:
        checksum = sum([ord(c) for c in byte_array]) % 0x100
    else:
        checksum = sum(byte_array) % 0x100

    return byte_array + _intarray2bytes([checksum])


class TemperatureUnit(Enum):
    C = 0x31
    F = 0x13


class ToneSet(Enum):
    PERMIT = 0x13
    NONE = 0x31


class StopButton(Enum):
    ENABLE = 0x13
    DISABLE = 0x31


class AlarmSetting(Enum):
    NONE = 0x00
    T3 = 0x03
    T10 = 0x0A


class WorkStatus(Enum):
    NOT_START = 0x00
    START = 0x01
    STOP = 0x02
    DELAY_START = 0x03


class RequestMessage:
    def to_bytes(self):
        pass


class ResponseMessage:
    def read(self, ser):
        """
        :type ser: serial.Serial
        """
        pass


class InitRequest(RequestMessage):
    def to_bytes(self):
        return _bin("CC 00 0A 00 D6")


class InitResponse(ResponseMessage):
    """
    :type msg: bytes
    """

    def __init__(self):
        self.msg = None

    def read(self, ser):
        """
        :type ser: serial.Serial
        """
        self.msg = ser.read(3)


class DevInfoRequest(RequestMessage):

    def to_bytes(self):
        return _bin("CC 00 06 00 D2")


class DevInfoResponse(ResponseMessage):
    """
    :type station_no: int
    :type rec_interval: time
    :type upper_limit: float
    :type lower_limit: float
    :type last_online: datetime
    :type work_sts: WorkStatus
    :type start_time: datetime
    :type stop_button: StopButton
    :type rec_count: int
    :type current:datetime
    :type user_info:str
    :type dev_num:str
    :type delay: float
    :type tone_set: ToneSet
    :type alarm: AlarmSetting
    :type temp_unit: TemperatureUnit
    :type temp_calibration: float
    """

    def __init__(self):
        self.station_no = None
        self.rec_interval = None
        self.upper_limit = None
        self.lower_limit = None
        self.last_online = None
        self.work_sts = None
        self.start_time = None
        self.stop_button = None
        self.rec_count = None
        self.current = None
        self.user_info = None
        self.dev_num = None
        self.delay = None
        self.tone_set = None
        self.alarm = None
        self.temp_unit = None
        self.temp_calibration = None

    def read(self, ser):
        """
        :type ser: serial.Serial
        """
        res = ser.read(160)

        (_, station_no, _, rec_interval, upper_limit, lower_limit, last_online, work_sts,
         start_time, stp_btn, _, rec_count, current, user_info, dev_num, delay, tone_set,
         alarm, temp_unit, temp_calib, _) = unpack(
            '>1s'
            'b'  # station no
            '3s'
            '3s'  # record interval hh mm ss
            'h'  # upper limit
            'h'  # lower limit
            '7s'  # last_online
            'b'  # work_status
            '7s'  # start_time
            'b'  # stopbutton permit=0x13, prohibit=0x31
            'b'
            'h'  # record_count
            '7s'  # current_time
            '100s'  # info
            '10s'  # device number
            'b'  # delaytime
            'b'  # tone set
            'b'  # alarm
            'b'  # temp unit
            'b'  # temp calibration
            '7s',
            res)

        self.station_no = station_no
        self.rec_interval = _interval_unpack(rec_interval)
        self.upper_limit = upper_limit / 10.0
        self.lower_limit = lower_limit / 10.0
        self.last_online = _datetime_unpack(last_online)
        self.work_sts = WorkStatus(work_sts)
        self.start_time = _datetime_unpack(start_time)
        self.stop_button = StopButton(stp_btn)
        self.rec_count = rec_count
        self.current = _datetime_unpack(current)
        self.user_info = user_info.decode("utf-8").rstrip("\x00")
        self.dev_num = dev_num.decode("utf-8")
        self.delay = int(delay / 16.0) + 0.5 * (delay % 16)
        self.tone_set = ToneSet(tone_set)
        self.alarm = AlarmSetting(alarm)
        self.temp_unit = TemperatureUnit(temp_unit)
        self.temp_calibration = temp_calib / 10.0

    def to_param_put(self):
        """
        convert dev_info to ParamPutRequest message.
        :rtype : ParamPutRequest
        """
        req = ParamPutRequest(self.station_no)
        req.rec_interval = self.rec_interval
        req.upper_limit = self.upper_limit
        req.lower_limit = self.lower_limit
        req.update_station_no = self.station_no
        req.stop_button = self.stop_button
        req.delay = self.delay
        req.tone_set = self.tone_set
        req.alarm = self.alarm
        req.temp_unit = self.temp_unit
        req.temp_calibration = self.temp_calibration

        return req


class ParamPutRequest(RequestMessage):
    """
    :type target_station_no: int
    :type rec_interval: time
    :type upper_limit: float
    :type lower_limit: float
    :type update_station_no: int
    :type stop_button: StopButton
    :type delay: float
    :type tone_set: ToneSet
    :type alarm: AlarmSetting
    :type temp_unit: TemperatureUnit
    :type temp_calibration: float
    """

    def __init__(self, target_station_no):
        self.target_station_no = target_station_no
        self.rec_interval = time(0, 10, 0)

        self.upper_limit = 60.0
        self.lower_limit = -30.0
        self.update_station_no = 1

        self.stop_button = StopButton.DISABLE
        self.delay = 0
        self.tone_set = ToneSet.NONE
        self.alarm = AlarmSetting.NONE
        self.temp_unit = TemperatureUnit.C
        self.temp_calibration = 0

    def to_bytes(self):
        write_bytes = pack(
            ">b"  # 0x33
            "b"  # target station no
            "2s"  # 0x0500
            "3s"  # record interval
            "h"  # upper limit
            "h"  # lower limit
            "b"  # update station no
            "b"  # stopbutton permit=0x13, prohibit=0x31
            "b"  # delaytime
            "b"  # tone set
            "b"  # alarm
            'b'  # temp unit
            'b'  # temp calibration
            "6s",
            0x33,
            self.target_station_no,
            _bin('05 00'),
            _interval_pack(self.rec_interval),
            int(self.upper_limit * 10.0),
            int(self.lower_limit * 10.0),
            self.update_station_no,
            self.stop_button.value,
            int(self.delay) * 16 + (5 if (self.delay % 1 != 0) else 0),
            self.tone_set.value,
            self.alarm.value,
            self.temp_unit.value,
            int(self.temp_calibration * 10.0),
            _bin('00 00 00 00 00 00'),
        )

        ba = _append_checksum(write_bytes)
        return ba


class ParamPutResponse(ResponseMessage):
    def __init__(self):
        self.msg = None

    def read(self, ser):
        """
        :type ser: serial.Serial
        """
        self.msg = ser.read(3)


class DataHeaderRequest(RequestMessage):
    def __init__(self, target_station_no):
        self.target_station_no = target_station_no

    def to_bytes(self):
        write_bytes = pack(
            ">b"    # 0x33
            "b"     # target station no
            "b"     # command: datahead 0x01
            "b",    # page number 0x00
            0x33, self.target_station_no, 0x01, 0x00)

        return _append_checksum(write_bytes)

class DataHeaderResponse(ResponseMessage):
    """
    :type rec_count: int
    :type start_time: datetime
    """
    def __init__(self):
        self.rec_count = 0
        self.start_time = None

    def read(self, ser):
        """
        :type ser: serial.Serial
        """
        res = ser.read(11)

        (_, rec_count, start_time, _) = unpack(
            '>1s'
            'h'  # record_count
            '7s'  # current_time
            'b',
            res)

        self.start_time = _datetime_unpack(start_time)
        self.rec_count = rec_count

class DataBodyRequest(RequestMessage):
    def __init__(self, target_station_no, page_num):
        self.target_station_no = target_station_no
        self.page_num = page_num

    def to_bytes(self):
        write_bytes = pack(
            ">b"    # 0x33
            "b"     # target station no
            "b"     # command databody 0x02
            "b",    # page number
            0x33, self.target_station_no, 0x02, self.page_num)

        return _append_checksum(write_bytes)

class DataBodyResponse(ResponseMessage):
    """
    :type count: int
    :type records: list
    :type start_time: datetime
    """
    def __init__(self, count):
        self.count = count
        self.records = None

    def read(self, ser):
        """
        :type ser: serial.Serial
        """
        res = ser.read(self.count * 2 + 2)  #data(2bytes)*count + (comand:0x55 + checksum)

        unpacked = unpack(
            '>1s'+('h'*self.count)+"b",
            res)

        self.records = unpacked[1:-1]






# coding: utf8

__author__ = 'civic'

import unittest
from io import BytesIO

from elitech.msg import (
    _bin,
    _datetime_unpack,
    _interval_unpack,
    _interval_pack,
    _append_checksum,
)
from elitech.msg import *
from six import (
    b,
)

class TestFunctions(unittest.TestCase):
    def test_bin(self):
        self.assertEqual(_bin("01 02 FF AB"), b'\x01\x02\xFF\xAB')
        self.assertEqual(_bin("01"), b'\x01')

    def test_datetime_unpack(self):
        self.assertEqual(_datetime_unpack(b'\x07\xDF\x05\x0E\x07\x38\x0E'), datetime(2015, 5, 14, 7, 56, 14))

    def test_interval_unpack(self):
        self.assertEqual(_interval_unpack(b'\x01\x02\x03'), time(1, 2, 3))

    def test_interval_pack(self):
        self.assertEqual(_interval_pack(time(1, 2,3 )), b'\x01\x02\x03')

    def test_append_checksum(self):
        self.assertEqual(_append_checksum(b'\x01\x02\x03'), b'\x01\x02\x03\x06')
        self.assertEqual(_append_checksum(b'\xF0\x0A\x09'), b'\xF0\x0A\x09\x03')

class TestMessages(unittest.TestCase):
    def test_InitRequest(self):
        self.assertEqual(InitRequest().to_bytes(), _bin("CC 00 0A 00 D6"))

    def test_InitResponse(self):
        res = InitResponse()
        res.read(BytesIO(_bin("01 02 03")))
        self.assertEqual(res.msg, b'\x01\x02\x03')

    def test_DevInfoRequest(self):
        self.assertEqual(DevInfoRequest().to_bytes(), _bin("CC 00 06 00 D2"))

    def test_DevInfoResponse(self):
        res = DevInfoResponse()
        res.read(BytesIO(_bin("55 02 01 28 0A 00 00 1E 02 58 FE D4 07 DF 05 0E "
                              "16 2F 04 02 07 DF 05 0E 07 38 0E 13 64 00 09 07 "
                              "DF 05 0E 16 2F 36 52 43 2D 34 20 44 61 74 61 20 "
                              "4C 6F 67 67 65 72 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 39 39 30 30 31 31 "
                              "32 32 33 33 11 31 00 31 F1 00 00 00 00 00 00 B3"
                              )))
        self.assertEqual(res.station_no, 2)
        self.assertEqual(res.rec_interval, time(0, 0, 30))
        self.assertEqual(res.upper_limit, 60.0)
        self.assertEqual(res.lower_limit, -30.0)
        self.assertEqual(res.last_online, datetime(2015, 5, 14, 22, 47, 4))
        self.assertEqual(res.work_sts, WorkStatus.STOP)
        self.assertEqual(res.start_time, datetime(2015,5,14, 7,56,14))
        self.assertEqual(res.stop_button, StopButton.ENABLE)
        self.assertEqual(res.rec_count, 9)
        self.assertEqual(res.current, datetime(2015,5,14,22,47,54))
        self.assertEqual(res.user_info, "RC-4 Data Logger")
        self.assertEqual(res.dev_num, "9900112233")
        self.assertEqual(res.delay, 1.5)
        self.assertEqual(res.tone_set, ToneSet.NONE)
        self.assertEqual(res.alarm, AlarmSetting.NONE)
        self.assertEqual(res.temp_unit, TemperatureUnit.C)
        self.assertEqual(res.temp_calibration, -1.5)

    def test_to_param_put(self):
        res = DevInfoResponse()
        res.read(BytesIO(_bin("55 02 01 28 0A 00 00 1E 02 58 FE D4 07 DF 05 0E "
                              "16 2F 04 02 07 DF 05 0E 07 38 0E 13 64 00 09 07 "
                              "DF 05 0E 16 2F 36 52 43 2D 34 20 44 61 74 61 20 "
                              "4C 6F 67 67 65 72 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 39 39 30 30 31 31 "
                              "32 32 33 33 11 31 00 31 F1 00 00 00 00 00 00 B3"
                              )))
        param_put = res.to_param_put()
        self.assertEqual(param_put.rec_interval, time(0, 0, 30))
        self.assertEqual(param_put.target_station_no, 2)
        self.assertEqual(param_put.update_station_no, 2)
        self.assertEqual(param_put.rec_interval, time(0, 0, 30))
        self.assertEqual(param_put.upper_limit, 60.0)
        self.assertEqual(param_put.lower_limit, -30.0)
        self.assertEqual(param_put.stop_button, StopButton.ENABLE)
        self.assertEqual(param_put.delay, 1.5)
        self.assertEqual(param_put.tone_set, ToneSet.NONE)
        self.assertEqual(param_put.alarm, AlarmSetting.NONE)
        self.assertEqual(param_put.temp_unit, TemperatureUnit.C)
        self.assertEqual(param_put.temp_calibration, -1.5)

    def test_ParamPutRequest(self):
        req = ParamPutRequest(2)
        req.rec_interval = time(0, 0, 30)
        req.update_station_no = 2
        req.stop_button = StopButton.ENABLE
        req.delay = 0
        req.temp_calibration = -1.5

        self.assertEqual(req.to_bytes(), _bin("33 02 05 00 00 00 1E 02 58 FE D4 02 13 00 31 00 "
                                               "31 F1 00 00 00 00 00 00 EC"))

    def test_ParamPutResponse(self):
        res = ParamPutResponse()
        res.read(BytesIO(_bin("01 02 03")))
        self.assertTrue(res.msg, b'\x01\x02\x03')

    def test_DataHeaderRequest(self):
        req = DataHeaderRequest(123)
        self.assertEqual(req.to_bytes(), _bin("33 7B 01 00 AF"))

    def test_DataHeaderResponse(self):
        res = DataHeaderResponse()
        res.read(BytesIO(_bin("55 0B 76 07 DF 05 0E 17 04 35 1F")))
        self.assertEqual(res.rec_count, 2934)
        self.assertEqual(res.start_time, datetime(2015, 5, 14, 23, 4, 53))

    def test_DataBodyRequest(self):
        req = DataBodyRequest(123, 5)
        self.assertEqual(req.to_bytes(), _bin("33 7B 02 05 B5"))

    def test_DataBodyResponse(self):
        res = DataBodyResponse(10)
        res.read(BytesIO(_bin("55 00 01 FF FF 00 03 00 04 00 05 00 06 00 07 00 08 00 09 00 0A 8C")))
        self.assertEqual(len(res.records), 10)
        self.assertEqual(res.records, (1, -1, 3, 4, 5, 6, 7, 8, 9, 10))

if __name__ == '__main__':
    unittest.main()



# coding: utf8

__author__ = 'civic'

import unittest
import elitech

from elitech.msg import _bin, _append_checksum
from elitech.msg import *
from datetime import timedelta
from io import BytesIO
import six

class DummySerial:
    def __init__(self, res, callback=None):
        if res:
            self.buf = BytesIO(res)
        else:
            self.buf = None
            self.callback = callback

        self.ba = None

    def write(self, ba):
        self.ba = ba

    def read(self, length):
        if self.buf:
            return self.buf.read(length)
        else:
            if six.PY2:
                return self.callback(length, [int(ord(b)) for b in self.ba])
            else:
                return self.callback(length, self.ba)

    def open(self):
        pass
    def close(self):
        pass

class DeviceTest(unittest.TestCase):
    def test_init(self):
        device = elitech.Device(None)
        device._ser = DummySerial(_bin("01 02 03"))

        res = device.init()
        self.assertEqual(res.msg, _bin('01 02 03'))

    def test_get_devinfo(self):
        device = elitech.Device(None)
        device._ser = DummySerial(_bin("55 02 01 28 0A 00 00 1E 02 58 FE D4 07 DF 05 0E "
                              "16 2F 04 02 07 DF 05 0E 07 38 0E 13 64 00 09 07 "
                              "DF 05 0E 16 2F 36 52 43 2D 34 20 44 61 74 61 20 "
                              "4C 6F 67 67 65 72 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 39 39 30 30 31 31 "
                              "32 32 33 33 11 31 00 31 F1 00 00 00 00 00 00 B3"
                              ))

        res = device.get_devinfo()
        self.assertEqual(res.station_no, 2)
        self.assertEqual(res.rec_interval, time(0, 0, 30))
        self.assertEqual(res.upper_limit, 60.0)
        self.assertEqual(res.lower_limit, -30.0)
        self.assertEqual(res.last_online, datetime(2015, 5, 14, 22, 47, 4))
        self.assertEqual(res.work_sts, WorkStatus.STOP)
        self.assertEqual(res.start_time, datetime(2015,5,14, 7,56,14))
        self.assertEqual(res.stop_button, StopButton.ENABLE)
        self.assertEqual(res.rec_count, 9)
        self.assertEqual(res.current, datetime(2015,5,14,22,47,54))
        self.assertEqual(res.user_info, "RC-4 Data Logger")
        self.assertEqual(res.dev_num, "9900112233")
        self.assertEqual(res.delay, 1.5)
        self.assertEqual(res.tone_set, ToneSet.NONE)
        self.assertEqual(res.alarm, AlarmSetting.NONE)
        self.assertEqual(res.temp_unit, TemperatureUnit.C)
        self.assertEqual(res.temp_calibration, -1.5)

    def test_update(self):
        device = elitech.Device(None)
        device._ser = DummySerial(_bin("01 02 03"))

        req = ParamPutRequest(1)
        res = device.update(req)
        self.assertEqual(res.msg, _bin("01 02 03"))

    def test_get_data_header(self):
        device = elitech.Device(None)
        device._ser = DummySerial(_bin("55 0B 76 07 DF 05 0E 17 04 35 1F"))
        res = device.get_data_header(1)

        self.assertEqual(res.rec_count, 2934)
        self.assertEqual(res.start_time, datetime(2015, 5, 14, 23, 4, 53))

    def test_get_data_body_1page(self):
        device = elitech.Device(None)
        def callback(length, ba):
            if ba[0] == 0xCC:
                # devinfo
                return _bin("55 01 01 28 0A 01 02 03 02 58 FE D4 07 DF 0A 01 "
                            "00 00 00 02 07 DF 0A 01 00 00 00 13 64 00 0A 07 "
                            "DF 05 0E 16 2F 36 52 43 2D 34 20 44 61 74 61 20 "
                            "4C 6F 67 67 65 72 00 00 00 00 00 00 00 00 00 00 "
                            "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                            "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                            "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                            "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                            "00 00 00 00 00 00 00 00 00 00 39 39 30 30 31 31 "
                            "32 32 33 33 11 31 00 31 F1 00 00 00 00 00 00 FF"
                            )
            elif ba[0] == 0x33 and ba[2] == 0x01:
                # 温度データヘッダ
                return _append_checksum(_bin("55 00 0A 07 DF 0A 01 00 00 00"))

            elif ba[0] == 0x33 and ba[2] == 0x02:
                # 温度データボディ
                if ba[3] == 0:
                    # 1ページ目10件
                    return _bin("55 00 01 FF FF 00 03 00 04 00 05 00 06 00 07 00 08 00 09 00 0A 8C")

            raise ValueError("invalid request data length")
        device._ser = DummySerial(None, callback=callback)

        devinfo = DevInfoResponse()
        devinfo.station_no = 1
        devinfo.rec_count = 10
        devinfo.rec_interval = time(1, 2, 3)
        devinfo.start_time = datetime(2015, 10, 1, 0, 0, 0)

        dt = timedelta(hours=1, minutes=2, seconds=3)

        res = device.get_data()
        self.assertEqual(len(res), 10)
        self.assertEqual(res, [
            (1, devinfo.start_time + dt*0, 0.1),
            (2, devinfo.start_time + dt*1, -0.1),
            (3, devinfo.start_time + dt*2, 0.3),
            (4, devinfo.start_time + dt*3, 0.4),
            (5, devinfo.start_time + dt*4, 0.5),
            (6, devinfo.start_time + dt*5, 0.6),
            (7, devinfo.start_time + dt*6, 0.7),
            (8, devinfo.start_time + dt*7, 0.8),
            (9, devinfo.start_time + dt*8, 0.9),
            (10, devinfo.start_time + dt*9, 1.0),
        ])

    def test_get_data_body_2page(self):
        """ 2ページのデータ取得が行われる
        """
        device = elitech.Device(None)

        def callback(length, ba):
            if ba[0] == 0xCC:
                # devinfo
                return _bin("55 01 01 28 0A 01 02 03 02 58 FE D4 07 DF 0A 01 "
                              "00 00 00 02 07 DF 0A 01 00 00 00 13 64 00 6E 07 "
                              "DF 05 0E 16 2F 36 52 43 2D 34 20 44 61 74 61 20 "
                              "4C 6F 67 67 65 72 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
                              "00 00 00 00 00 00 00 00 00 00 39 39 30 30 31 31 "
                              "32 32 33 33 11 31 00 31 F1 00 00 00 00 00 00 FF"
                              )
            elif ba[0] == 0x33 and ba[2] == 0x01:
                # 温度データヘッダ
                return _append_checksum(_bin("55 00 6E 07 DF 0A 01 00 00 00"))
            elif ba[0] == 0x33 and ba[2] == 0x02:
                # 温度データボディ
                if ba[3] == 0:
                    # 1ページ目100件
                    return _bin("55 00 00 00 01 00 02 00 03 00 04 00 05 00 06 00 07 00 08 00 09 00 0A 00 0B 00 0C 00 0D "
                                "00 0E 00 0F 00 10 00 11 00 12 00 13 00 14 00 15 00 16 00 17 00 18 00 19 00 1A 00 1B 00 "
                                "1C 00 1D 00 1E 00 1F 00 20 00 21 00 22 00 23 00 24 00 25 00 26 00 27 00 28 00 29 00 2A "
                                "00 2B 00 2C 00 2D 00 2E 00 2F 00 30 00 31 00 32 00 33 00 34 00 35 00 36 00 37 00 38 00 "
                                "39 00 3A 00 3B 00 3C 00 3D 00 3E 00 3F 00 40 00 41 00 42 00 43 00 44 00 45 00 46 00 47 "
                                "00 48 00 49 00 4A 00 4B 00 4C 00 4D 00 4E 00 4F 00 50 00 51 00 52 00 53 00 54 00 55 00 "
                                "56 00 57 00 58 00 59 00 5A 00 5B 00 5C 00 5D 00 5E 00 5F 00 60 00 61 00 62 00 63 AB")
                elif ba[3] == 1:
                    # 2ページ目10件
                    return _bin("55 00 64 00 65 00 66 00 67 00 68 00 69 00 6A 00 6B 00 6C 00 6D 6A")
            raise ValueError("invalid request data length")

        device._ser = DummySerial(None, callback=callback)

        devinfo = DevInfoResponse()
        devinfo.station_no = 1
        devinfo.rec_count = 110
        devinfo.rec_interval = time(1, 2, 3)
        devinfo.start_time = datetime(2015, 10, 1, 0, 0, 0)

        dt = timedelta(hours=1, minutes=2, seconds=3)

        res = device.get_data()
        self.assertEqual(len(res), 110)

        expect = []
        for n in range(110):
            expect.append((n+1, devinfo.start_time + dt*n, n / 10.0))

        self.assertEqual(res, expect)





from setuptools import setup,find_packages
import os
import setuptools.command.sdist

class SdistCommand(setuptools.command.sdist.sdist):
    def run(self):
        try:
            import pypandoc

            rst = pypandoc.convert("README.md", "rst")
            with open("README.txt", "w") as f:
                f.write(rst)
            print("convert markdown to rst")
        except:
            pass

        super(SdistCommand, self).run()
        os.remove("README.txt")


setup(
    cmdclass={'sdist': SdistCommand},
    name='elitech-datareader',
    version='0.8.2',
    packages=find_packages(),
    url='http://github.com/civic/elitech-datareader/',
    license='MIT',
    author='civic',
    author_email='dev@civic-apps.com',
    description='Elitech rc4 data access tool and library.',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Communications',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    platforms='any',
    test_suite = "tests",
    install_requires=open('requirements.txt').read().splitlines(),
    scripts = ['scripts/elitech-device.py'],
    dependency_links=["http://www.silabs.com/products/mcu/Pages/USBtoUARTBridgeVCPDrivers.aspx"]

)



__author__ = 'civic'

from serial import Serial
import time
from datetime import (
    timedelta
)
import serial
import math

from .msg import (
    AlarmSetting,
    StopButton,
    TemperatureUnit,
    ToneSet,
    WorkStatus,
    RequestMessage,
    ResponseMessage,
    InitRequest,
    InitResponse,
    DevInfoRequest,
    DevInfoResponse,
    ParamPutRequest,
    ParamPutResponse,
    DataHeaderRequest,
    DataHeaderResponse,
    DataBodyRequest,
    DataBodyResponse,
)
import six

class Device:
    def __init__(self, serial_port):
        if serial_port is not None:
            self._ser = serial.Serial(serial_port, 115200, timeout=3)
            self._ser.close()
        self.debug = False
        self.wait_time = 0.5

    def _talk(self, request, response):
        """
        :type request: RequestMessage
        """
        ba = request.to_bytes()

        if (self.debug):
            print("\nba length={}".format(len(ba)))
            for i, b in enumerate(ba):
                if six.PY2:
                    six.print_("{:02X} ".format(ord(b)), sep='', end='')
                else:
                    six.print_("{:02X} ".format(b), end='')
                if (i + 1) % 16 == 0:
                    six.print_()
            six.print_()

        self._ser.write(ba)

        response.read(self._ser)

        return response

    def init(self):
        """
        :rtype: InitResponse
        """
        req = InitRequest()

        try:
            self._ser.open()
            res = self._talk(req, InitResponse())
        finally:
            self._ser.close()
            time.sleep(self.wait_time)

        return res

    def get_devinfo(self):
        """
        :rtype: DevInfoResponse
        """
        req = DevInfoRequest()
        try:
            self._ser.open()
            res = self._talk(req, DevInfoResponse())
        finally:
            self._ser.close()
            time.sleep(self.wait_time)

        return res

    def update(self, req):
        """
        :type req: ParamPutRequest
        :rtype: ParamPutResponse
        """
        try:
            self._ser.open()
            res = self._talk(req, ParamPutResponse())
        finally:
            self._ser.close()
            time.sleep(self.wait_time)

        return res

    def get_data_header(self, target_station_no):
        """
        :rtype: DataHeaderResponse
        """
        try:
            self._ser.open()
            req = DataHeaderRequest(target_station_no)
            res = self._talk(req, DataHeaderResponse())
        finally:
            self._ser.close()
            time.sleep(self.wait_time)

        return res

    def get_data(self, callback=None):
        """
        :type devinfo: DevInfoResponse
        :rtype:list[(int,datetime,float)]
        """
        devinfo = self.get_devinfo()
        header = self.get_data_header(devinfo.station_no)

        page = int(math.ceil(header.rec_count / 100.0))
        dt = timedelta(hours=devinfo.rec_interval.hour,
                      minutes=devinfo.rec_interval.minute,
                      seconds=devinfo.rec_interval.second)

        data_list = []
        base_time = devinfo.start_time
        no = 1
        try:
            self._ser.open()
            for p in range(page):

                req = DataBodyRequest(devinfo.station_no, p)
                count = 100 if (p+1) * 100 < devinfo.rec_count else (devinfo.rec_count % 100)
                res = DataBodyResponse(count)
                self._talk(req, res)

                for rec in res.records:
                    data_list.append((no, base_time, rec/10.0))
                    no += 1
                    base_time += dt
                if callback is not None:
                    callback(data_list)
                    data_list = []
        finally:
            self._ser.close()
            time.sleep(self.wait_time)

        return data_list
